"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = adapterComponent;

var _sfc = _interopRequireDefault(require("./sfc"));

var _renderHelpers = _interopRequireDefault(require("render-helpers"));

var _mixin = require("./mixin");

var _convertComponentsDefining = _interopRequireDefault(require("./convertComponentsDefining"));

var _lifecyclesMap = _interopRequireDefault(require("./lifecyclesMap"));

var _nextTick = _interopRequireDefault(require("./nextTick"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function adapterComponent(defining, renderFactory, styles, Rax) {
  // Set Rax as render
  _renderHelpers.default._r(Rax);

  if (defining.components) {
    /**
     * Compatible to commonjs2
     */
    Object.keys(defining.components).forEach(function (componentName) {
      var componentDeclear = defining.components[componentName];

      if (componentDeclear.__esModule === true && typeof componentDeclear.default === 'function') {
        defining.components[componentName] = componentDeclear.default;
      }
    });
    /**
     * Convert component defining name
     * - MyComponentName(PascalCase) => my-component-name(kebab-case)
     */

    (0, _convertComponentsDefining.default)(defining.components);
  } else {
    /**
     * Must have a components object scope for doing
     *   _c(components['view'] || 'view')
     */
    defining.components = {};
  }

  var isCSSTextMode = typeof styles === 'string';
  var renderLifecycle = renderFactory(styles, defining, _renderHelpers.default, _utils.isWeex);
  return (
    /*#__PURE__*/
    function (_Rax$Component) {
      _inherits(_class, _Rax$Component);

      function _class(props, context) {
        var _this;

        _classCallCheck(this, _class);

        _this = _possibleConstructorReturn(this, _getPrototypeOf(_class).call(this, props, context));
        var vm = _this.vm = new _sfc.default(_objectSpread({}, defining, {
          propsData: props,
          children: props.children
        }));
        (0, _mixin.proxy)(vm, '$refs', _assertThisInitialized(_assertThisInitialized(_this)), 'refs'); // Bind mount related life cycles to Rax.

        var fns = Object.keys(_lifecyclesMap.default);

        for (var i = 0, l = fns.length; i < l; i++) {
          if (typeof defining[fns[i]] === 'function') {
            _this[_lifecyclesMap.default[fns[i]]] = defining[fns[i]].bind(vm);
          }
        } // Fire destroyed at next tick


        if (typeof defining.destroyed === 'function') {
          var prevComponentWillUnmount = _this.componentWillUnmount;

          _this.componentWillUnmount = function () {
            prevComponentWillUnmount && prevComponentWillUnmount.call(this);
            (0, _nextTick.default)(defining.destroyed, vm);
          };
        }

        var prevComponentWillReceiveProps = _this.componentWillReceiveProps;

        _this.componentWillReceiveProps = function (nextProps) {
          prevComponentWillReceiveProps && prevComponentWillReceiveProps.apply(vm, arguments);

          if (nextProps !== this.props && vm.hasOwnProperty('_props')) {
            // update propsData
            var keys = Object.keys(nextProps);

            for (var _i = 0, _l = keys.length; _i < _l; _i++) {
              var prop = keys[_i];

              if (vm._props.hasOwnProperty(prop) && vm._props[prop] !== nextProps[prop]) {
                vm[prop] = nextProps[prop];
              }
            }
          }

          if (nextProps.children !== this.props.children) {
            // update slots
            (0, _mixin.mixinSlots)(vm, nextProps.children);
          }
        };
        /* @overrides forceUpdate */


        vm.forceUpdate = _this.forceUpdate.bind(_assertThisInitialized(_assertThisInitialized(_this)));
        /* @overrides render */

        _this.render = function () {
          var VTree = renderLifecycle.call(vm); // Inject style tag, if CSS text mode enables.

          if (isCSSTextMode && styles.length > 0) {
            return [Rax.createElement('style', null, styles), VTree];
          } else {
            return VTree;
          }
        };

        return _this;
      }

      return _class;
    }(Rax.Component)
  );
}