"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = initWatch;

var _utils = require("./utils");

var _watcher = _interopRequireDefault(require("./watcher"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initWatch(vm, watch) {
  Object.defineProperty(vm, '$watch', {
    enumerable: false,
    value: $watch
  });

  if (!(0, _utils.isPlainObject)(watch)) {
    return;
  }

  for (var key in watch) {
    var handler = watch[key];

    if (Array.isArray(handler)) {
      for (var i = 0; i < handler.length; i++) {
        createWatcher(vm, key, handler[i]);
      }
    } else {
      createWatcher(vm, key, handler);
    }
  }
}

function createWatcher(vm, expOrFn, handler, options) {
  if ((0, _utils.isPlainObject)(handler)) {
    options = handler;
    handler = handler.handler;
  }

  if (typeof handler === 'string') {
    handler = vm[handler];
  }

  return vm.$watch(expOrFn, handler, options);
}

function $watch(expOrFn, cb, options) {
  var vm = this;

  if ((0, _utils.isPlainObject)(cb)) {
    return createWatcher(vm, expOrFn, cb, options);
  }

  options = options || {};
  options.user = true;
  var watcher = new _watcher.default(vm, expOrFn, cb, options);

  if (options.immediate) {
    cb.call(vm, watcher.value);
  }

  return function unwatchFn() {
    watcher.teardown();
  };
}