"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = mixinComputed;

var _utils = require("./utils");

var _watcher = _interopRequireDefault(require("./watcher"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var sharedPropertyDefinition = {
  enumerable: true,
  configurable: true,
  get: _utils.noop,
  set: _utils.noop
};

function mixinComputed(vm, computed) {
  if (!(0, _utils.isPlainObject)(computed)) {
    return;
  }

  var watchers = vm._computedWatchers = Object.create(null);

  for (var key in computed) {
    var userDef = computed[key];
    var getter = typeof userDef === 'function' ? userDef : userDef.get; // create internal watcher for the computed property.

    watchers[key] = new _watcher.default(vm, getter || _utils.noop, _utils.noop, {
      computed: true
    }); // component-defined computed properties are already defined on the
    // component prototype. We only need to define computed properties defined
    // at instantiation here.

    if (!(key in vm)) {
      defineComputed(vm, key, userDef);
    } else if (process.env.NODE_ENV !== 'production') {
      if (key in vm.$data) {
        (0, _utils.warn)("The computed property \"".concat(key, "\" is already defined in data."), vm);
      } else if (vm.$options.props && key in vm.$options.props) {
        (0, _utils.warn)("The computed property \"".concat(key, "\" is already defined as a prop."), vm);
      }
    }
  }
}

function defineComputed(target, key, userDef) {
  if (typeof userDef === 'function') {
    sharedPropertyDefinition.get = createComputedGetter(key);
    sharedPropertyDefinition.set = _utils.noop;
  } else {
    sharedPropertyDefinition.get = userDef.get ? userDef.cache !== false ? createComputedGetter(key) : userDef.get : _utils.noop;
    sharedPropertyDefinition.set = userDef.set ? userDef.set : _utils.noop;
  }

  if (process.env.NODE_ENV !== 'production' && sharedPropertyDefinition.set === _utils.noop) {
    sharedPropertyDefinition.set = function () {
      (0, _utils.warn)("Computed property \"".concat(key, "\" was assigned to but it has no setter."), this);
    };
  }

  Object.defineProperty(target, key, sharedPropertyDefinition);
}

function createComputedGetter(key) {
  return function computedGetter() {
    var watcher = this._computedWatchers && this._computedWatchers[key];

    if (watcher) {
      watcher.depend();
      return watcher.evaluate();
    }
  };
}