"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toggleUpdate = toggleUpdate;
exports.observeWithContext = observeWithContext;
exports.default = mixinData;

var _observe = _interopRequireDefault(require("./observe"));

var _nextTick = _interopRequireDefault(require("./nextTick"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var allowUpdate = true;

function toggleUpdate(allow) {
  var type = _typeof(allow);

  if (type === 'undefined') {
    allowUpdate = !allowUpdate;
  } else if (type === 'boolean') {
    allowUpdate = allow;
  }
}

function observeWithContext(data, context, def) {
  var isDirty = false;

  function update() {
    (0, _nextTick.default)(function () {
      if (allowUpdate && isDirty) {
        context.forceUpdate();
        isDirty = false;
      }
    }, context);
  }

  (0, _observe.default)(data, {
    afterSetter: function afterSetter() {
      isDirty = true;
      update();
    }
  });
}

function mixinData(context, declearation) {
  // init reactive data
  var data = context.data; // data always exists

  var __data_type = _typeof(declearation.data);

  if (__data_type === 'function') {
    data = declearation.data.call(context);
  } else if (__data_type === 'object' && process.env.NODE_ENV !== 'production') {
    /* istanbul ignore next */
    console.error('[SFC Loader WARN]:', 'The "data" option should be a function that returns a per-instance value in component definitions.');
    data = declearation.data;
  } else {
    /* istanbul ignore next */
    data = {};
  }

  context._data = data; // $data getter

  Object.defineProperty(context, '$data', {
    get: function get() {
      return data;
    }
  });

  if (!data) {
    return;
  }

  observeWithContext(data, context, declearation);
  Object.keys(data).forEach(function (dataKey) {
    context[dataKey] = data[dataKey];
    Object.defineProperty(context, dataKey, {
      enumerable: true,
      configurable: true,
      get: function proxyGet() {
        return data[dataKey];
      },
      set: function proxySet(newVal) {
        data[dataKey] = newVal;
      }
    });
  });
}