"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = mixinProps;

var _utils = require("./utils");

var _observe = require("./observe");

function mixinProps(vm) {
  var propsData = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var propsDefining = arguments.length > 2 ? arguments[2] : undefined;
  var _props = {};
  Object.defineProperty(vm, '$props', {
    get: function get() {
      return _props;
    }
  }); // if no propsDefining in config, no props will be added

  if (!propsDefining) return;
  var formattedPropsDefining = [];

  if (Array.isArray(propsDefining)) {
    propsDefining.forEach(function (propKey) {
      if (typeof propKey === 'string') {
        formattedPropsDefining.push({
          key: propKey
        });
      } else if (process.env.NODE_ENV !== 'production') {
        (0, _utils.warn)('props key should be a string, check definition of', propsDefining);
      }
    });
  } else if ((0, _utils.isPlainObject)(propsDefining)) {
    var keys = Object.keys(propsDefining);

    for (var i = 0, l = keys.length; i < l; i++) {
      var key = keys[i];
      var defaultVal = propsDefining[key].default;
      formattedPropsDefining.push({
        key: key,
        defaultVal: defaultVal
      });
    }
  }

  vm._props = _props;

  for (var _i = 0, _l = formattedPropsDefining.length; _i < _l; _i++) {
    var _formattedPropsDefini = formattedPropsDefining[_i],
        _key = _formattedPropsDefini.key,
        _defaultVal = _formattedPropsDefini.defaultVal;

    if (propsData.hasOwnProperty(_key)) {
      _props[_key] = propsData[_key];
    } else {
      _props[_key] = _defaultVal;
    }

    (0, _observe.defineReactive)(vm, _key, _props[_key], {
      afterSetter: function afterSetter() {
        vm.forceUpdate();
      }
    });
  }
}