"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = mixinSlots;

// null, default or slotName
function getSlotName(item) {
  if (item && Object.hasOwnProperty.call(item, 'props')) {
    return item.props.slot || 'default';
  } else {
    return 'default';
  }
}

function deleteSlot(item) {
  if (item && Object.hasOwnProperty.call(item, 'props')) {
    delete item.props.slot;
  }
}

function mixinSlots(vm, children) {
  var $slots = {};

  function injectSlot(child) {
    var slotName = getSlotName(child);

    if (null === slotName) {
      return;
    }

    $slots[slotName] = $slots[slotName] || [];
    $slots[slotName].push(child); // remove slot attr to avoid disappear

    deleteSlot(child);
  }

  if (Array.isArray(children)) {
    for (var i = 0, l = children.length; i < l; i++) {
      injectSlot(children[i]);
    }
  } else {
    injectSlot(children);
  }

  Object.defineProperty(vm, '$slots', {
    enumerable: false,
    configurable: true,
    get: function get() {
      return $slots;
    }
  });
}