"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = nextTick;

var _utils = require("./utils");

// next tick
var UA = typeof navigator === 'undefined' ? '' : navigator && navigator.userAgent;
var isWMLIOS = typeof __windmill_environment__ !== 'undefined' && __windmill_environment__.platform === 'iOS';
/* eslint-disable-line */

var isIOS = UA && /iphone|ipad|ipod|ios/.test(UA) || isWMLIOS;
var callbacks = [];
var pending = false;

var timerFunc = function timerFunc() {
  setTimeout(flushCallbacks, 0);
};

if (typeof Promise !== 'undefined' && (0, _utils.isNative)(Promise)) {
  var p = Promise.resolve();

  timerFunc = function timerFunc() {
    p.then(flushCallbacks); // in problematic UIWebViews, Promise.then doesn't completely break, but
    // it can get stuck in a weird state where callbacks are pushed into the
    // microtask queue but the queue isn't being flushed, until the browser
    // needs to do some other work, e.g. handle a timer. Therefore we can
    // "force" the microtask queue to be flushed by adding an empty timer.

    if (isIOS) {
      setTimeout(function () {});
    }
  };
}

function flushCallbacks() {
  pending = false;
  var copies = callbacks.slice(0);
  callbacks.length = 0;

  for (var i = 0; i < copies.length; i++) {
    copies[i]();
  }
}

function nextTick(cb, ctx) {
  var _resolve;

  callbacks.push(function () {
    if (cb) {
      try {
        cb.call(ctx);
      } catch (e) {
        console.error(e, ctx, 'nextTick');
      }
    } else if (_resolve) {
      _resolve(ctx);
    }
  });

  if (!pending) {
    pending = true;
    timerFunc();
  }

  if (!cb) {
    return new Promise(function (resolve) {
      _resolve = resolve;
    });
  }
}