"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defineReactive = defineReactive;
exports.default = void 0;

var _utils = require("./utils");

var _const = require("./const");

var _Dep = _interopRequireDefault(require("./Dep"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var arrayProto = Array.prototype;

function createArrayDelegate(opts) {
  var arrayMethods = Object.create(arrayProto);

  _const.arrayPatchMethods.forEach(function (method) {
    // hijack
    var originalMethod = arrayProto[method];
    Object.defineProperty(arrayMethods, method, {
      value: function value() {
        var ob = this.__ob__;

        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        var result = originalMethod.apply(this, args);
        var inserted;

        switch (method) {
          case 'push':
          case 'unshift':
            inserted = args;
            break;

          case 'splice':
            inserted = args.slice(2);
            break;
        }

        if (inserted) {
          ob.observeArray(inserted);
        }

        if (typeof opts.afterSetter === 'function') {
          opts.afterSetter(this);
        } // notify change


        ob.dep.notify();
        return result;
      },
      enumerable: true,
      writable: true,
      configurable: true
    });
  });

  return arrayMethods;
}

function observe(value, options) {
  if (!(0, _utils.isObject)(value)) {
    return;
  }

  if (value && Object.hasOwnProperty.call(value, '__ob__') && value.__ob__ instanceof Observer) {
    return value.__ob__;
  }

  return new Observer(value, options);
}

var Observer =
/*#__PURE__*/
function () {
  function Observer(value, options) {
    _classCallCheck(this, Observer);

    this.value = value;
    this.dep = new _Dep.default();
    Object.defineProperty(value, '__ob__', {
      value: this,
      enumerable: false,
      writable: true,
      configurable: true
    });

    if (Array.isArray(value)) {
      // delegate array methods
      value.__proto__ = createArrayDelegate(options);
      this.observeArray(value, options);
    } else if ((0, _utils.isPlainObject)(value)) {
      this.walk(value, options);
    }
  }

  _createClass(Observer, [{
    key: "walk",
    value: function walk(obj, opt) {
      var keys = Object.keys(obj);

      for (var i = 0; i < keys.length; i++) {
        defineReactive(obj, keys[i], undefined, opt);
      }
    }
  }, {
    key: "observeArray",
    value: function observeArray(items, options) {
      for (var i = 0, l = items.length; i < l; i++) {
        observe(items[i], options);
      }
    }
  }]);

  return Observer;
}();
/**
 * vue observe function
 */


function defineReactive(obj, key, val) {
  var opts = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var dep = new _Dep.default();
  var property = Object.getOwnPropertyDescriptor(obj, key);

  if (property && property.configurable === false) {
    return;
  } // cater for pre-defined getter/setters


  var getter = property && property.get;
  var setter = property && property.set;

  if ((!getter || setter) && typeof val === 'undefined') {
    val = obj[key];
  }

  var parents = [];

  if (Array.isArray(opts.parents)) {
    parents = parents.concat(opts.parents);
  }

  parents.push(key);
  var children = observe(val, _objectSpread({}, opts, {
    parents: parents
  }));
  Object.defineProperty(obj, key, {
    enumerable: true,
    configurable: true,
    get: function reactiveGetter() {
      var value = getter ? getter.call(obj) : val;

      if (_Dep.default.target) {
        dep.depend();

        if (children) {
          children.dep.depend();

          if (Array.isArray(value)) {
            dependArray(value);
          }
        }
      }

      return value;
    },
    set: function reactiveSetter(newVal) {
      var value = getter ? getter.call(obj) : val; // 值不变的时候不触发

      if (newVal === value || newVal !== newVal && value !== value) {
        return;
      }

      dep.notify();

      if (setter) {
        setter.call(obj, newVal);
      } else {
        val = newVal;
      }

      if (typeof opts.afterSetter === 'function') {
        opts.afterSetter(newVal);
      }

      children = observe(newVal, _objectSpread({}, opts, {
        parents: parents
      }));
      dep.notify();
    }
  });
}
/**
 * Collect dependencies on array elements when the array is touched, since
 * we cannot intercept array element access like property getters.
 */


function dependArray(value) {
  for (var e, i = 0, l = value.length; i < l; i++) {
    e = value[i];
    e && e.__ob__ && e.__ob__.dep.depend();

    if (Array.isArray(e)) {
      dependArray(e);
    }
  }
}

var _default = observe;
exports.default = _default;