"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _Dep = _interopRequireWildcard(require("./Dep"));

var _scheduler = require("./scheduler");

var _utils = require("./utils");

var _Set = _interopRequireDefault(require("./Set"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var uid = 0;

var Watcher =
/*#__PURE__*/
function () {
  function Watcher(vm, expOrFn, cb, options, isRenderWatcher) {
    _classCallCheck(this, Watcher);

    this.vm = vm;

    if (isRenderWatcher) {
      vm._watcher = this;
    }

    vm._watchers.push(this); // options


    if (options) {
      this.deep = !!options.deep;
      this.user = !!options.user;
      this.computed = !!options.computed;
      this.sync = !!options.sync;
      this.before = options.before;
    } else {
      this.deep = this.user = this.computed = this.sync = false;
    }

    this.cb = cb;
    this.id = ++uid;
    this.active = true;
    this.dirty = this.computed; // for computed watchers

    this.deps = [];
    this.newDeps = [];
    this.depIds = new _Set.default();
    this.newDepIds = new _Set.default();
    this.expression = process.env.NODE_ENV !== 'production' ? expOrFn.toString() : ''; // parse expression for getter

    if (typeof expOrFn === 'function') {
      this.getter = expOrFn;
    } else {
      this.getter = (0, _utils.parsePath)(expOrFn);

      if (!this.getter) {
        this.getter = function () {};
      }
    }

    if (this.computed) {
      this.value = undefined;
      this.dep = new _Dep.default();
    } else {
      this.value = this.get();
    }
  }
  /**
   * Evaluate the getter, and re-collect dependencies.
   */


  _createClass(Watcher, [{
    key: "get",
    value: function get() {
      (0, _Dep.pushTarget)(this);
      var value;
      var vm = this.vm;

      try {
        value = this.getter.call(vm, vm);
      } catch (e) {
        if (this.user) {
          console.error(e, vm, "getter for watcher \"".concat(this.expression, "\""));
        } else {
          throw e;
        }
      } finally {
        // "touch" every property so they are all tracked as
        // dependencies for deep watching
        // if (this.deep) {
        //   traverse(value)
        // }
        (0, _Dep.popTarget)();
        this.cleanupDeps();
      }

      return value;
    }
    /**
     * Add a dependency to this directive.
     */

  }, {
    key: "addDep",
    value: function addDep(dep) {
      var id = dep.id;

      if (!this.newDepIds.has(id)) {
        this.newDepIds.add(id);
        this.newDeps.push(dep);

        if (!this.depIds.has(id)) {
          dep.addSub(this);
        }
      }
    }
    /**
     * Clean up for dependency collection.
     */

  }, {
    key: "cleanupDeps",
    value: function cleanupDeps() {
      var i = this.deps.length;

      while (i--) {
        var dep = this.deps[i];

        if (!this.newDepIds.has(dep.id)) {
          dep.removeSub(this);
        }
      }

      var tmp = this.depIds;
      this.depIds = this.newDepIds;
      this.newDepIds = tmp;
      this.newDepIds.clear();
      tmp = this.deps;
      this.deps = this.newDeps;
      this.newDeps = tmp;
      this.newDeps.length = 0;
    }
    /**
     * Scheduler job interface.
     * Will be called by the scheduler.
     */

  }, {
    key: "run",
    value: function run() {
      if (this.active) {
        this.getAndInvoke(this.cb);
      }
    }
    /**
     * Subscriber interface.
     * Will be called when a dependency changes.
     */

  }, {
    key: "update",
    value: function update() {
      var _this = this;

      if (this.computed) {
        // A computed property watcher has two modes: lazy and activated.
        // It initializes as lazy by default, and only becomes activated when
        // it is depended on by at least one subscriber, which is typically
        // another computed property or a component's render function.
        if (this.dep.subs.length === 0) {
          // In lazy mode, we don't want to perform computations until necessary,
          // so we simply mark the watcher as dirty. The actual computation is
          // performed just-in-time in this.evaluate() when the computed property
          // is accessed.
          this.dirty = true;
        } else {
          // In activated mode, we want to proactively perform the computation
          // but only notify our subscribers when the value has indeed changed.
          this.getAndInvoke(function () {
            _this.dep.notify();
          });
        }
      } else if (this.sync) {
        this.run();
      } else {
        (0, _scheduler.queueWatcher)(this);
      }
    }
  }, {
    key: "getAndInvoke",
    value: function getAndInvoke(cb) {
      var value = this.get();

      if (value !== this.value || // Deep watchers and watchers on Object/Arrays should fire even
      // when the value is the same, because the value may
      // have mutated.
      (0, _utils.isObject)(value) || this.deep) {
        // set new value
        var oldValue = this.value;
        this.value = value;
        this.dirty = false;

        if (this.user) {
          try {
            cb.call(this.vm, value, oldValue);
          } catch (e) {
            console.error(e, this.vm, "callback for watcher \"".concat(this.expression, "\""));
          }
        } else {
          cb.call(this.vm, value, oldValue);
        }
      }
    }
    /**
     * Evaluate and return the value of the watcher.
     * This only gets called for computed property watchers.
     */

  }, {
    key: "evaluate",
    value: function evaluate() {
      if (this.dirty) {
        this.value = this.get();
        this.dirty = false;
      }

      return this.value;
    }
    /**
     * Depend on this watcher. Only for computed property watchers.
     */

  }, {
    key: "depend",
    value: function depend() {
      if (this.dep && _Dep.default.target) {
        this.dep.depend();
      }
    }
    /**
     * Remove self from all dependencies' subscriber list.
     */

  }, {
    key: "teardown",
    value: function teardown() {
      if (this.active) {
        // remove self from vm's watcher list
        // this is a somewhat expensive operation so we skip it
        // if the vm is being destroyed.
        if (!this.vm._isBeingDestroyed) {
          var watchers = this.vm._watchers;

          if (watchers.length) {
            var index = watchers.indexOf(this);

            if (index > -1) {
              return watchers.splice(index, 1);
            }
          }
        }

        var i = this.deps.length;

        while (i--) {
          this.deps[i].removeSub(this);
        }

        this.active = false;
      }
    }
  }]);

  return Watcher;
}();

exports.default = Watcher;